@@ -64,8 +64,7 @@ with other DBs.
 
 use v5.12;
 
-package SQL::Steno;
-our $VERSION = 'v0.2.2';	# quotes because of CPAN indexer bug
+package SQL::Steno v0.2.2.1;
 
 use strict;
 use Time::HiRes qw(gettimeofday tv_interval);
@@ -551,8 +550,12 @@ my %esc = map { $_ eq 'v' ? "\013" : eval( qq!"\\$_"! ), "\\$_" } qw'0 a b e f n
     }
 }
 
-sub render_csv($;$) {
+sub render_csv($;$$) {
     my( $sth, $filter ) = @_;
+    my( $semi, $tab ) =
+	(exists $_[2]{semi},
+	 exists $_[2]{tab})
+	if $_[2];
     my $name = $sth->{NAME};
     my @row = @$name;
     while() {
@@ -561,14 +564,14 @@ sub render_csv($;$) {
 		$_ = qq!"$_"! if
 		    /\A\Z/ or
 		    s/"/""/g or
-		    tr/,\n// or
+		    $semi ? tr/;\n// : $tab ? tr/\t\n// : tr/,\n// or
 		    /\A=/;
 	    } else {
 		$_ = '';
 	    }
 	    utf8::decode $_;
 	}
-	print join( ',', @row ) . "\n";
+	print join( $semi ? ';' : $tab ? "\t" : ',', @row ) . "\n";
 
       FETCH:
 	@row = $sth->fetchrow_array
@@ -577,8 +580,16 @@ sub render_csv($;$) {
     }
 }
 
-sub render_tabular($;$) {
+sub render_tabular($;$$) {
     my( $sth, $filter ) = @_;
+    my( $null, $crlf, $date ) =
+	exists $_[2]{all} ?
+	('NULL', 1, 1) :
+	(exists $_[2]{NULL} ? 'NULL' : exists $_[2]{null} ? 'null' : 0,
+	 exists $_[2]{crlf},
+	 exists $_[2]{date})
+	if $_[2];
+    $null ||= '\@';
     my @name = @{$sth->{NAME}};
     my @len = (0) x @name;
     my( @txt, @res, @comp );
@@ -586,15 +597,16 @@ sub render_tabular($;$) {
 	next if $filter && !$filter->( \@name, @res1 );
 	for my $i ( 0..$#res1 ) {
 	    if( !defined $res1[$i] ) {
-		$res1[$i] = '\@';
+		$res1[$i] = $null;
 	    } elsif( $res1[$i] !~ /^\d+(?:\.\d+)?$/ ) {
 		$txt[$i] = 1;
-		$res1[$i] =~ s/\r\n/\\R/g;
+		$res1[$i] =~ s/\r\n/\\R/g unless $crlf;
 		$res1[$i] =~ s/([\t\n\r])/$esc{$1}/g;
 		no warnings 'uninitialized';
 		$res1[$i] =~ s/^(?:(0000-)00-00|(1970-)01-01)(?:( 00:)00:00)?$/$1$2$3/ or
 		    $res1[$i] =~ s/^(\d{4}-\d\d-\d\d )?(?:00:00:00|23:59:5(9))$/$1 . ($2 ? '24:' : '00:')/e or
-			$res1[$i] =~ s/^((\d{4}-\d\d-\d\d )?\d\d:\d\d):00$/$1/;
+		    $res1[$i] =~ s/^((\d{4}-\d\d-\d\d )?\d\d:\d\d):00$/$1/
+		    unless $date;
 		utf8::decode $res1[$i];
 	    }
 	    $txt[$i] = 0 if @txt < $i;
@@ -649,7 +661,8 @@ sub render_tabular($;$) {
 
 my $yaml_re = join '', sort keys %esc;
 $yaml_re =~ s!\\!\\\\!;
-sub render_yaml($;$) {
+my $tabsize = $ENV{TABSIZE} || 8;
+sub render_yaml($;$$) {
     my( $sth, $filter ) = @_;
     my @label;			       # Fill on 0th round with same transformation as data (but \n inline)
     my $count = count $DBI::rows || 1; # init \todo don't know how many unfiltered
@@ -666,7 +679,7 @@ sub render_yaml($;$) {
 		s/([$yaml_re])/$esc{$1}/go;
 		s/([\0-\010\013-\037\177-\237])/sprintf "\\x%02x", ord $1/ge;
 		$_ = qq!"$_"!;
-	    } elsif( tr/\n//d ) {
+	    } elsif( tr/\n// ) {
 		my $nl = chomp;
 		s/^/    /mg;
 		substr $_, 0, 0, $nl ? "|2\n" : "|2-\n";
@@ -678,10 +691,14 @@ sub render_yaml($;$) {
 	} else {
 	    my $maxlen = 0;
 	    for( @row ) {
-		substr $_, 0, 0, $maxlen ? '  ' : '- ';
-		$maxlen = length if $maxlen < length;
+		substr $_, 0, 0, $maxlen ? '  ' : '- '; # 1st field if no maxlen yet
+		my $length = 0;
+		$length += $1 ? $tabsize - $length % $tabsize : length $2
+		    while /\G(?:(\t)|([^\t]+))/gc;
+		$_ .= ": $length";
+		$maxlen = $length if $maxlen < $length;
 	    }
-	    $_ .= ': ' . ' ' x ($maxlen - length)
+	    s/(\d+)\Z/' ' x ($maxlen - $1)/e
 		for @label = @row;
 	}
       FETCH:
